/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { fakeAsync, TestBed, tick } from '@angular/core/testing';
import { AbstractControl, UntypedFormBuilder, UntypedFormGroup } from '@angular/forms';
import { UserAddFormService } from './user-add-form.service';
import SpyInstance = jest.SpyInstance;

describe('UserAddFormService', () => {
  let formService: UserAddFormService;
  let alfaGroup: UntypedFormGroup;

  beforeEach(() => {
    TestBed.configureTestingModule({
      providers: [UserAddFormService, UntypedFormBuilder],
    });

    formService = TestBed.inject(UserAddFormService);
    alfaGroup = <UntypedFormGroup>formService.form.get(UserAddFormService.ROLLEN_GROUP).get(UserAddFormService.ALFA_GROUP);
  });

  it('should create', () => {
    expect(formService).toBeTruthy();
  });

  describe('initAlfaGroupLogic', () => {
    it('should call handleAlfaGroupChange when value of form element changes', fakeAsync(() => {
      const handleAlfaGroupChangeSpy: SpyInstance = jest.spyOn(formService as any, 'handleAlfaGroupChange');

      alfaGroup.get(UserAddFormService.LOESCHEN).setValue(true);

      tick();

      expect(handleAlfaGroupChangeSpy).toHaveBeenCalled();
    }));
  });

  describe('handleAlfaGroupChange', () => {
    it('should call disableUncheckedCheckboxes if any checkbox is checked', () => {
      jest.spyOn(formService as any, 'isAnyChecked').mockReturnValue(true);
      const disableUncheckedCheckboxesSpy: SpyInstance = jest.spyOn(formService as any, 'disableUncheckedCheckboxes');

      formService.handleAlfaGroupChange(alfaGroup);

      expect(disableUncheckedCheckboxesSpy).toHaveBeenCalled();
    });

    it('should call enableAllCheckboxes if not any checkbox is checked', () => {
      jest.spyOn(formService as any, 'isAnyChecked').mockReturnValue(false);
      const enableAllCheckboxesSpy: SpyInstance = jest.spyOn(formService as any, 'enableAllCheckboxes');

      formService.handleAlfaGroupChange(alfaGroup);

      expect(enableAllCheckboxesSpy).toHaveBeenCalled();
    });
  });

  describe('isAnyChecked', () => {
    it('should return false if no checkbox is checked', () => {
      const result = formService.isAnyChecked(alfaGroup);

      expect(result).toBe(false);
    });

    it('should return true if any checkbox is checked', () => {
      alfaGroup.get(UserAddFormService.LOESCHEN).setValue(true);

      const result = formService.isAnyChecked(alfaGroup);

      expect(result).toBe(true);
    });
  });

  describe('disableUncheckedCheckboxes', () => {
    it('if control value is false then control should be disabled', () => {
      const control: AbstractControl = alfaGroup.get(UserAddFormService.LOESCHEN);
      control.setValue(false);

      formService.disableUncheckedCheckboxes(alfaGroup);

      expect(control.disabled).toBe(true);
    });

    it('if control value is true then control should NOT be disabled', () => {
      const control: AbstractControl = alfaGroup.get(UserAddFormService.LOESCHEN);
      control.setValue(true);

      formService.disableUncheckedCheckboxes(alfaGroup);

      expect(control.disabled).toBe(false);
    });
  });

  describe('updateCheckboxStates', () => {
    it('if control value is false then control should be disabled', () => {
      const control: AbstractControl = alfaGroup.get(UserAddFormService.LOESCHEN);
      control.setValue(false);

      formService.disableUncheckedCheckboxes(alfaGroup);

      expect(control.disabled).toBe(true);
    });
  });

  describe('enableAllCheckboxes', () => {
    it('if control value is true then control should be enabled', () => {
      const control: AbstractControl = alfaGroup.get(UserAddFormService.LOESCHEN);
      const enableSpy = jest.spyOn(control, 'enable');

      formService.enableAllCheckboxes(alfaGroup);

      expect(enableSpy).toHaveBeenCalled();
    });
  });
});
