/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { BescheidService } from '@alfa-client/bescheid-shared';
import { CommandResource } from '@alfa-client/command-shared';
import { StateResource } from '@alfa-client/tech-shared';
import {
  createDialogRefMock,
  DialogRefMock,
  getElementComponentFromFixtureByCss,
  mock,
  Mock,
  triggerEvent,
} from '@alfa-client/test-utils';
import { OzgcloudStrokedButtonWithSpinnerComponent } from '@alfa-client/ui';
import { VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { createSuccessfullyDoneCommandStateResource } from '../../../../../../command-shared/test/command';
import { getDataTestIdOf } from '../../../../../../tech-shared/test/data-test';
import { singleColdCompleted } from '../../../../../../tech-shared/test/marbles';
import { createDialogResult } from '../../../../../../ui/src/lib/ui/ozgcloud-dialog/ozgcloud-dialog.result';
import { createVorgangWithEingangResource } from '../../../../../../vorgang-shared/test/vorgang';
import {
  AbschliessenDialogData,
  BescheidWizardAbschliessenDialogContainerComponent,
} from './bescheid-wizard-abschliessen-dialog-container.component';

describe('BescheidWizardAbschliessenDialogContainerComponent', () => {
  let component: BescheidWizardAbschliessenDialogContainerComponent;
  let fixture: ComponentFixture<BescheidWizardAbschliessenDialogContainerComponent>;

  const closeBescheidButton: string = getDataTestIdOf('close-bescheid-button');
  const confirmButtonDataTestId: string = getDataTestIdOf('ueberspringen-abschliessen-button');
  const cancelButtonDataTestId: string = getDataTestIdOf('ueberspringen-abbrechen-button');
  const vorgangWithEingangResource: VorgangWithEingangResource = createVorgangWithEingangResource();
  const dialogData: AbschliessenDialogData = { vorgangWithEingangResource };

  let dialogRef: DialogRefMock;
  let bescheidService: Mock<BescheidService>;

  beforeEach(() => {
    bescheidService = mock(BescheidService);
    dialogRef = createDialogRefMock();
  });

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        BescheidWizardAbschliessenDialogContainerComponent,
        MockComponent(MatIcon),
        MockComponent(OzgcloudStrokedButtonWithSpinnerComponent),
      ],
      providers: [
        {
          provide: DIALOG_DATA,
          useValue: dialogData,
        },
        {
          provide: DialogRef,
          useValue: dialogRef,
        },
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(BescheidWizardAbschliessenDialogContainerComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('component', () => {
    describe('onClose', () => {
      it('should close dialog', () => {
        component.onClose();

        expect(dialogRef.close).toHaveBeenCalled();
      });
    });

    describe('onConfirm', () => {
      it('should call bescheid service', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        bescheidService.bescheidErstellungUeberspringen.mockReturnValue(of(commandStateResource));

        component.onConfirm();

        expect(bescheidService.bescheidErstellungUeberspringen).toHaveBeenCalledWith(vorgangWithEingangResource);
      });

      it('should set abschliessen$', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        bescheidService.bescheidErstellungUeberspringen.mockReturnValue(of(commandStateResource));

        component.onConfirm();

        expect(component.abschliessen$).toBeObservable(singleColdCompleted(commandStateResource));
      });

      it('should close dialog', () => {
        const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
        bescheidService.bescheidErstellungUeberspringen.mockReturnValue(of(commandStateResource));

        component.onConfirm();
        component.abschliessen$.subscribe();

        expect(dialogRef.close).toHaveBeenCalledWith(createDialogResult(commandStateResource));
      });
    });
  });

  describe('template', () => {
    describe('button close', () => {
      describe('output', () => {
        it('should call onClose', () => {
          component.onClose = jest.fn();

          triggerEvent({ fixture, name: 'click', elementSelector: closeBescheidButton });

          expect(component.onClose).toHaveBeenCalled();
        });
      });
    });

    describe('ozgcloud-stroked-button-with-spinner Überspringen und abschließen', () => {
      function getElementComponent(): OzgcloudStrokedButtonWithSpinnerComponent {
        return getElementComponentFromFixtureByCss(fixture, confirmButtonDataTestId);
      }

      describe('input', () => {
        it('should set stateResource', () => {
          const commandStateResource: StateResource<CommandResource> = createSuccessfullyDoneCommandStateResource();
          component.abschliessen$ = of(commandStateResource);

          fixture.detectChanges();

          expect(getElementComponent().stateResource).toBe(commandStateResource);
        });
      });

      describe('output', () => {
        it('should call onConfirm', () => {
          component.onConfirm = jest.fn();

          triggerEvent({
            fixture,
            name: 'click',
            elementSelector: confirmButtonDataTestId,
          });

          expect(component.onConfirm).toHaveBeenCalled();
        });
      });
    });

    describe('ozgcloud-stroked-button-with-spinner Abbrechen', () => {
      it('should call onClose', () => {
        component.onClose = jest.fn();

        triggerEvent({ fixture, name: 'click', elementSelector: cancelButtonDataTestId });

        expect(component.onClose).toHaveBeenCalled();
      });
    });
  });
});
