/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import {
  StateResource,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { SnackBarService } from '@alfa-client/ui';
import { HttpErrorResponse, HttpStatusCode } from '@angular/common/http';
import { faker } from '@faker-js/faker';
import { Store } from '@ngrx/store';
import { Resource } from '@ngxp/rest';
import { cold, hot } from 'jest-marbles';
import {
  createCommand,
  createCommandErrorResource,
  createCommandListResource,
  createCommandResource,
  createCreateCommandProps,
} from 'libs/command-shared/test/command';
import { createHttpErrorResponse } from 'libs/tech-shared/test/http';
import { toResource } from 'libs/tech-shared/test/resource';
import { Observable, Subject, of, throwError } from 'rxjs';
import { CommandLinkRel } from './command.linkrel';
import { CommandErrorMessage } from './command.message';
import {
  Command,
  CommandListResource,
  CommandOrder,
  CommandResource,
  CreateCommandProps,
} from './command.model';
import { CommandRepository } from './command.repository';
import {
  CommandService,
  IntervallHandleWithTickObservable,
  startInterval,
} from './command.service';

import * as Actions from './+state/command.actions';
import * as Selectors from './+state/command.selectors';

describe('CommandService', () => {
  let service: CommandService;
  let repository: Mock<CommandRepository>;
  let store: Mock<Store>;
  let selectionSubject: Subject<any>;

  const snackbarService: Mock<SnackBarService> = mock(SnackBarService);

  const command: Command = createCommand();

  const commandResource: CommandResource = createCommandResource();
  const commandStateResource: StateResource<CommandResource> = createStateResource(commandResource);

  const commandResourceWithUpdateLink: CommandResource = createCommandResource([
    CommandLinkRel.UPDATE,
  ]);

  beforeEach(() => {
    store = mock(Store);
    selectionSubject = new Subject();
    store.select.mockReturnValue(selectionSubject);
    store.dispatch = jest.fn();

    repository = mock(CommandRepository);
    service = new CommandService(
      useFromMock(repository),
      useFromMock(snackbarService),
      useFromMock(<any>store),
    );
  });

  describe('create command', () => {
    const resource: Resource = toResource({});
    const linkRel: string = faker.word.sample();

    beforeEach(() => {
      repository.createCommand.mockReturnValue(cold('a', { a: commandResourceWithUpdateLink }));
    });

    it('should call repository with resource, linkrel and command', () => {
      service.createCommand(resource, linkRel, command);

      expect(repository.createCommand).toHaveBeenCalledWith(resource, linkRel, command);
    });

    it('should call handleHttpError', () => {
      const errorResponse: HttpErrorResponse = createHttpErrorResponse();
      repository.createCommand.mockReturnValue(throwError(errorResponse));
      service.handleHttpError = jest.fn();

      service.createCommand(resource, linkRel, command).subscribe();

      expect(service.handleHttpError).toHaveBeenCalledWith(errorResponse);
    });

    it('should call handleCommand', () => {
      repository.createCommand.mockReturnValue(of(commandResource));
      service.handleCommand = jest.fn();

      service.createCommand(resource, linkRel, command).subscribe();

      expect(service.handleCommand).toHaveBeenCalledWith(commandResource);
    });
  });

  describe('revoke command', () => {
    beforeEach(() => {
      repository.revokeCommand.mockReturnValue(cold('a', { a: commandResource }));
    });

    it('should call repository with resource, linkrel and command', () => {
      service.revokeCommand(commandResource);

      expect(repository.revokeCommand).toHaveBeenCalledWith(commandResource);
    });

    it('should call handleHttpError', () => {
      const errorResponse: HttpErrorResponse = createHttpErrorResponse();
      repository.revokeCommand.mockReturnValue(throwError(errorResponse));
      service.handleHttpError = jest.fn();

      service.revokeCommand(commandResource).subscribe();

      expect(service.handleHttpError).toHaveBeenCalledWith(errorResponse);
    });

    it('should call handleCommand', () => {
      repository.revokeCommand.mockReturnValue(of(commandResource));
      service.handleCommand = jest.fn();

      service.revokeCommand(commandResource).subscribe();

      expect(service.handleCommand).toHaveBeenCalledWith(commandResource);
    });

    describe('process command with status in progress(with update link)', () => {
      beforeEach(() => {
        repository.revokeCommand.mockReturnValue(cold('a', { a: commandResourceWithUpdateLink }));
        service.pollCommand = jest.fn();
        (<any>service.pollCommand).mockReturnValue(
          cold('a', { a: createStateResource(commandResourceWithUpdateLink, true) }),
        );
      });

      it('should return value with loading true', () => {
        const result = service.revokeCommand(commandResource);

        expect(result).toBeObservable(
          hot('a', { a: createStateResource(commandResourceWithUpdateLink, true) }),
        );
      });
    });

    describe('process command with status done(no update link)', () => {
      it('should return value with loading false', () => {
        const result = service.revokeCommand(commandResource);

        expect(result).toBeObservable(hot('a', { a: commandStateResource }));
      });
    });
  });

  describe('handleCommand', () => {
    it.skip('should call handleCommandError', () => {
      service.handleCommandError = jest.fn();
      const commandWithError: CommandResource = createCommandErrorResource();
      service.handleCommand(commandWithError);

      expect(service.handleCommandError).toHaveBeenCalledWith(commandWithError);
    });

    it('should call startPolling', () => {
      const commandWithoutError: CommandResource = {
        ...createCommandResource(),
        errorMessage: null,
      };
      service.startPolling = jest.fn();
      service.handleCommand(commandWithoutError);

      expect(service.startPolling).toHaveBeenCalledWith(commandWithoutError);
    });

    describe.skip('handleCommandError', () => {
      describe('on concurrent modifiation error', () => {
        it('should call snackBarService on concurrentModification error', () => {
          service.handleCommandError({
            ...commandResource,
            errorMessage: CommandErrorMessage.CONCURRENT_MODIFICATION,
          });

          expect(snackbarService.showError).toHaveBeenCalledWith(
            'Der Vorgang wurde zwischenzeitlich verändert und wurde neu geladen.',
          );
        });

        it('should dispatch "dispatchConcurrentModificationCommand" action', () => {
          service.handleCommandError({
            ...commandResource,
            errorMessage: CommandErrorMessage.CONCURRENT_MODIFICATION,
          });

          expect(store.dispatch).toHaveBeenCalledWith(Actions.publishConcurrentModificationAction);
        });
      });

      it('should return command as stateResource', () => {
        service.handleCommandError(commandResource).subscribe(() => {
          expect(commandStateResource).toEqual(createStateResource(commandResource));
        });
      });
    });
  });

  describe('handle interval', () => {
    const interval: IntervallHandleWithTickObservable = { handle: 0, tickObservable: of(null) };

    beforeEach(() => {
      service.clearInterval = jest.fn();
    });

    it('should call clearInterval', () => {
      service.handleInterval(createEmptyStateResource(), interval);

      expect(service.clearInterval).toHaveBeenCalledWith(interval.handle);
    });

    it('should NOT call clearInterval', () => {
      service.handleInterval(createEmptyStateResource(true), interval);

      expect(service.clearInterval).not.toHaveBeenCalled();
    });
  });

  describe('getAndUpdate', () => {
    beforeEach(() => {
      repository.getCommand.mockReturnValue(cold('a', { a: commandResourceWithUpdateLink }));
    });

    it('should call getCommand', () => {
      service.getAndUpdate(commandResource);

      expect(repository.getCommand).toHaveBeenCalled();
    });

    it('should return stateResource still loading', () => {
      const result = service.getAndUpdate(commandResource);

      expect(result).toBeObservable(
        hot('a', { a: createStateResource(commandResourceWithUpdateLink, true) }),
      );
    });

    describe('command is loaded', () => {
      beforeEach(() => {
        repository.getCommand.mockReturnValue(cold('a', { a: commandResource }));
      });

      it('should loaded stateResource', () => {
        const result = service.getAndUpdate(commandResource);

        expect(result).toBeObservable(hot('a', { a: createStateResource(commandResource) }));
      });
    });
  });

  describe('get command', () => {
    beforeEach(() => {
      repository.getCommand.mockReturnValue(cold('a', { a: commandResource }));
    });

    it('should call repository with resource', () => {
      service.getCommand(commandResource);

      expect(repository.getCommand).toHaveBeenCalledWith(commandResource);
    });

    it('should return value', () => {
      const result = service.getCommand(commandResource);

      expect(result).toBeObservable(hot('a', { a: commandResource }));
    });
  });

  describe('get effected resource', () => {
    const listResource = {};
    const command: CommandResource = createCommandResource();

    beforeEach(() => {
      repository.getEffectedResource.mockReturnValue(cold('a', { a: listResource }));
    });

    it('should call repository', () => {
      service.getEffectedResource(command);

      expect(repository.getEffectedResource).toHaveBeenCalledWith(command);
    });

    it('should return value', () => {
      const result = service.getEffectedResource(command);

      expect(result).toBeObservable(hot('a', { a: listResource }));
    });
  });

  describe('get pending commands', () => {
    const commandListResource: CommandListResource = createCommandListResource();

    beforeEach(() => {
      repository.getPendingCommands.mockReturnValue(cold('a', { a: commandListResource }));
    });

    it('should call repository', () => {
      service.getPendingCommands(commandResource, CommandLinkRel.SELF);

      expect(repository.getPendingCommands).toHaveBeenLastCalledWith(
        commandResource,
        CommandLinkRel.SELF,
      );
    });
  });

  describe('start interval', () => {
    const intervall: number = 100;

    it('should not be null', () => {
      const result = startInterval(intervall);

      expect(result).not.toBeNull();
    });

    it('should have handle not null', () => {
      const result = startInterval(intervall);

      expect(result.handle).not.toBeNull();
    });

    it('should have tick as observable', () => {
      const result = startInterval(intervall);

      expect(result.tickObservable).toBeInstanceOf(Observable);
    });
  });

  describe('handleErrorByStatus', () => {
    const errorResponse: HttpErrorResponse = createHttpErrorResponse();

    it('should return error state resource on status 422', () => {
      const result = service.handleErrorByStatus({
        ...errorResponse,
        status: HttpStatusCode.UnprocessableEntity,
      });

      expect(result).toEqual(createErrorStateResource(errorResponse.error));
    });
  });

  describe('createCommandByProps', () => {
    const createCommandProps: CreateCommandProps = { ...createCreateCommandProps(), command };

    beforeEach(() => {
      service.getCommandByOrder = jest.fn().mockReturnValue(of(commandStateResource));
    });

    it('should dispatch action', () => {
      service.createCommandByProps(createCommandProps);

      expect(store.dispatch).toHaveBeenCalledWith(Actions.createCommand(createCommandProps));
    });

    it('should call get command by order', (done) => {
      service.createCommandByProps(createCommandProps).subscribe(() => {
        expect(service.getCommandByOrder).toHaveBeenCalledWith(command.order);
        done();
      });

      selectionSubject.next(commandStateResource);
    });

    it('should return selected command', (done) => {
      service.createCommandByProps(createCommandProps).subscribe((selected) => {
        expect(selected).toBe(commandStateResource);
        done();
      });

      selectionSubject.next(commandStateResource);
    });
  });

  describe('get command by order', () => {
    it('should select from store', (done) => {
      const selectorSpy = jest.spyOn(Selectors, 'commandByOrder');

      service.getCommandByOrder(CommandOrder.VORGANG_ANNEHMEN).subscribe(() => {
        expect(selectorSpy).toHaveBeenCalledWith(CommandOrder.VORGANG_ANNEHMEN);
        done();
      });

      selectionSubject.next(commandStateResource);
    });

    it('should return value', (done) => {
      service.getCommandByOrder(CommandOrder.VORGANG_ANNEHMEN).subscribe((selected) => {
        expect(selected).toBe(commandStateResource);
        done();
      });

      selectionSubject.next(commandStateResource);
    });

    it('should not emit on null', (done) => {
      store.select.mockReturnValue(of(null));

      let success: boolean = true;
      service.getCommandByOrder(<CommandOrder>'anyOrder').subscribe({
        next: (commandStateResource: StateResource<CommandResource>) => {
          success = commandStateResource !== null;
        },
        complete: () => {
          expect(success).toBeTruthy();
          done();
        },
      });
    });
  });
});
