/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.Link;
import org.springframework.hateoas.RepresentationModel;
import org.springframework.hateoas.mediatype.hal.HalModelBuilder;
import org.springframework.hateoas.server.RepresentationModelAssembler;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.common.binaryfile.BinaryFileController;
import de.ozgcloud.alfa.common.user.UserManagerUrlProvider;
import de.ozgcloud.alfa.postfach.PostfachMail.Direction;
import de.ozgcloud.alfa.postfach.PostfachMailController.PostfachMailCommandByVorgangController;
import de.ozgcloud.alfa.postfach.PostfachMailController.PostfachMailCommandController;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHead;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
class PostfachMailModelAssembler implements RepresentationModelAssembler<PostfachMail, EntityModel<PostfachMail>> {

	public static final String REL_SEND_POSTFACH_MAIL = "sendPostfachMail";
	public static final String REL_SEND = "send";
	public static final String REL_RESEND_POSTFACH_MAIL = "resendPostfachMail";
	public static final String REL_RESET_NEW_POSTFACH_MAIL = "resetHasNewPostfachNachricht";
	public static final String REL_ATTACHMENTS = "attachments";
	public static final String REL_EDIT = "edit";
	static final String REL_UPLOAD_ATTACHMENT = "uploadAttachment";
	static final String REL_CREATED_BY = "createdBy";
	private static final String POSTFACH_NACHRICHT_ATTACHMENT_FIELD = "postfachNachrichtAttachment";
	private static final String HAS_NEW_POSTFACH_NACHRICHT_FIELD = "hasNewPostfachNachricht";
	private static final String FILE_PATH = "file";

	private static final Predicate<PostfachMail> IS_OUTGOING = postfachNachricht -> postfachNachricht.getDirection() == Direction.OUT;

	private static final Predicate<PostfachMail> SENT_FAILED = postfachMail -> Objects.nonNull(postfachMail.getSentAt())
			&& BooleanUtils.isFalse(postfachMail.getSentSuccessful());
	private static final Predicate<PostfachMail> IS_UNSEND = IS_OUTGOING.and(mail -> !mail.getSentSuccessful() && Objects.isNull(mail.getSentAt()));
	private static final Predicate<PostfachMail> HAS_ATTACHMENTS = nachricht -> CollectionUtils.isNotEmpty(nachricht.getAttachments());

	static final String SYSTEM_USER_IDENTIFIER = "system";
	private static final Predicate<PostfachMail> SENT_BY_CLIENT_USER = postfachNachricht -> Optional.ofNullable(postfachNachricht.getCreatedBy())
			.map(createdBy -> !createdBy.toString().startsWith(SYSTEM_USER_IDENTIFIER)).orElse(false);

	private final UserManagerUrlProvider userManagerUrlProvider;

	public RepresentationModel<EntityModel<PostfachSettings>> toCollectionModel(Stream<PostfachMail> postfachMails, VorgangWithEingang vorgang,
			PostfachSettings postfachSettings) {

		var model = buildHalRepresentationModel(postfachMails, vorgang, postfachSettings);

		if (hasServiceKonto(vorgang)) {
			addPostfachNachrichtLinks(model, vorgang);
		}
		return model;
	}

	RepresentationModel<EntityModel<PostfachSettings>> buildHalRepresentationModel(Stream<PostfachMail> postfachMails, VorgangWithEingang vorgang,
			PostfachSettings postfachSettings) {
		return HalModelBuilder
				.halModelOf(postfachSettings)
				.link(linkTo(methodOn(PostfachMailController.class).getAll(vorgang.getId())).withSelfRel())
				.embed(postfachMails.map(this::toModel))
				.build();
	}

	boolean hasServiceKonto(VorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getHeader()).map(VorgangHead::getServiceKonto).isPresent();
	}

	void addPostfachNachrichtLinks(RepresentationModel<EntityModel<PostfachSettings>> model, VorgangWithEingang vorgang) {
		var vorgangId = vorgang.getId();

		if (vorgang.getStatus() != VorgangStatus.ZU_LOESCHEN) {
			model.add(linkTo(methodOn(PostfachMailCommandByVorgangController.class).sendPostfachMail(vorgangId, null))
					.withRel(REL_SEND_POSTFACH_MAIL));
		}

		model.add(linkTo(BinaryFileController.class).slash(vorgangId).slash(POSTFACH_NACHRICHT_ATTACHMENT_FIELD).slash(FILE_PATH)
				.withRel(REL_UPLOAD_ATTACHMENT));

		if (vorgang.isHasNewPostfachNachricht()) {
			model.add(linkTo(VorgangController.class).slash(vorgangId).slash(HAS_NEW_POSTFACH_NACHRICHT_FIELD).withRel(REL_RESET_NEW_POSTFACH_MAIL));
		}
	}

	@Override
	public EntityModel<PostfachMail> toModel(PostfachMail postfachMail) {
		var selfLink = linkTo(PostfachMailController.class).slash(postfachMail.getId());

		return ModelBuilder.fromEntity(postfachMail).addLink(selfLink.withSelfRel())
				.ifMatch(IS_UNSEND).addLink(linkTo(PostfachMailCommandController.class, postfachMail.getId()).withRel(REL_SEND))
				.ifMatch(SENT_FAILED)
				.addLink(linkTo(methodOn(PostfachMailCommandController.class).createCommand(postfachMail.getId(), null))
						.withRel(REL_RESEND_POSTFACH_MAIL))
				.ifMatch(IS_UNSEND)
				.addLink(linkTo(methodOn(PostfachMailCommandController.class).createCommand(postfachMail.getId(), null))
						.withRel(REL_EDIT))
				.ifMatch(HAS_ATTACHMENTS)
				.addLink(linkTo(methodOn(PostfachMailController.class).findAttachments(PostfachNachrichtId.from(postfachMail.getId())))
						.withRel(REL_ATTACHMENTS))
				.ifMatch(() -> userManagerUrlProvider.isConfiguredForUserProfile() && SENT_BY_CLIENT_USER.test(postfachMail))
				.addLink(() -> Link.of(String.format(userManagerUrlProvider.getUserProfileTemplate(), postfachMail.getCreatedBy()),
						REL_CREATED_BY))
				.buildModel();
	}

}