/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.collaboration;

import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.hateoas.EntityModel;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import lombok.SneakyThrows;

class CollaborationControllerTest {

	@InjectMocks
	private CollaborationController controller;

	@Mock
	private CollaborationService service;

	@Mock
	private CollaborationModelAssembler assembler;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@Nested
	class TestGetById {

		private final CollaborationAssertions collaborationAssertions = new CollaborationAssertions(false);

		@Nested
		class OnOrganisationsEinheitCollaboration {

			public static final String COLLABORATION_ID = OrganisationsEinheitCollaborationTestFactory.ID;
			private final Collaboration collaboration = OrganisationsEinheitCollaborationTestFactory.create();

			@BeforeEach
			void init() {
				when(service.getById(OrganisationsEinheitCollaborationTestFactory.ID)).thenReturn(collaboration);
				when(assembler.toModel(collaboration)).thenReturn(EntityModel.of(collaboration));
			}

			@Test
			void shouldCallCollaborationService() {
				performRequest();

				verify(service).getById(COLLABORATION_ID);
			}

			@Test
			void shouldCreateModel() {
				performRequest();

				verify(assembler).toModel(collaboration);
			}

			@SneakyThrows
			@Test
			void shouldReturnStatusOk() {
				var response = performRequest();

				response.andExpect(status().isOk());
			}

			@SneakyThrows
			@Test
			void shouldHaveContentWithFields() {
				var response = performRequest();

				collaborationAssertions.assertHasFields(response);
			}

			@SneakyThrows
			@Test
			void shouldHaveZustaendigeStelleLink() {
				var response = performRequest();

				collaborationAssertions.assertHasZustaendigeStelleLink(response, OrganisationsEinheitController.PATH,
						OrganisationsEinheitCollaborationTestFactory.ZUSTAENDIGE_STELLE);
			}

			@SneakyThrows
			private ResultActions performRequest() {
				return mockMvc.perform(get(CollaborationController.PATH + "/" + COLLABORATION_ID));
			}
		}

		@Nested
		class OnFachstelleCollaboration {

			public static final String COLLABORATION_ID = FachstelleCollaborationTestFactory.ID;
			private final Collaboration collaboration = FachstelleCollaborationTestFactory.create();

			@BeforeEach
			void init() {
				when(service.getById(FachstelleCollaborationTestFactory.ID)).thenReturn(collaboration);
				when(assembler.toModel(collaboration)).thenReturn(EntityModel.of(collaboration));
			}

			@SneakyThrows
			@Test
			void shouldHaveContentWithLink() {
				var response = performRequest();

				collaborationAssertions.assertHasZustaendigeStelleLink(response, FachstelleController.PATH,
						FachstelleCollaborationTestFactory.ZUSTAENDIGE_STELLE);
			}

			@SneakyThrows
			private ResultActions performRequest() {
				return mockMvc.perform(get(CollaborationController.PATH + "/" + COLLABORATION_ID));
			}
		}
	}
}
