/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.binaryfile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.List;
import java.util.Map;

import jakarta.validation.ConstraintValidatorContext;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import com.thedeanda.lorem.LoremIpsum;

class UploadBinaryFileContentTypeValidatorTest {

	@Mock
	private BinaryFileProperties properties;
	@Spy
	@InjectMocks
	private UploadBinaryFileContentTypeValidator validator;

	@Nested
	class TestIsValid {

		@Mock
		private ConstraintValidatorContext context;

		@Test
		void shouldGetConfiguredContentTypes() {
			callValidator();

			verify(validator).getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);
		}

		@Test
		void shouldReturnTrueIfConfigurationIsMissing() {
			var result = callValidator();

			assertThat(result).isTrue();
		}

		@Test
		void shouldReturnFalseIfContentTypeIsNotConfigured() {
			givenConfiguredContentTypes(List.of("dummy/abc", "dummy/xyz"));

			var result = callValidator();

			assertThat(result).isFalse();
		}

		@Test
		void shouldReturnTrueIfContentTypeIsConfigured() {
			givenConfiguredContentTypes(List.of(BinaryFileTestFactory.CONTENT_TYPE));

			var result = callValidator();

			assertThat(result).isTrue();
		}

		private void givenConfiguredContentTypes(List<String> contentTypes) {
			doReturn(contentTypes).when(validator).getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);
		}

		private boolean callValidator() {
			return validator.isValid(UploadBinaryFileTestFactory.create(), context);
		}
	}

	@Nested
	class TestGetConfiguredContentTypes {

		@Test
		void shouldReturnEmptyCollection() {
			var contentTypes = validator.getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);

			assertThat(contentTypes).isEmpty();
		}

		@Test
		void shouldReturnContentTypes() {
			var expectedContentTypes = List.of(LoremIpsum.getInstance().getWords(1), LoremIpsum.getInstance().getWords(1));
			givenConfiguredContentTypes(expectedContentTypes);

			var contentTypes = validator.getConfiguredContentTypes(UploadBinaryFileTestFactory.FIELD);

			assertThat(contentTypes).containsExactlyInAnyOrderElementsOf(expectedContentTypes);
		}

		private void givenConfiguredContentTypes(List<String> contentTypes) {
			when(properties.getContentTypes()).thenReturn(Map.of(UploadBinaryFileTestFactory.FIELD, contentTypes));
		}
	}
}
