/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.hint;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.Collections;
import java.util.List;
import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.alfa.common.user.CurrentUserService;
import de.ozgcloud.alfa.common.user.UserProfileTestFactory;
import de.ozgcloud.alfa.common.user.UserRole;
import de.ozgcloud.alfa.vorgang.ZustaendigeStelleTestFactory;

class HintServiceTest {

	@Spy
	@InjectMocks
	private HintService service;
	@Mock
	private CurrentUserService currentUserService;

	@DisplayName("Find hints")
	@Nested
	class TestFindHints {

		@Test
		void shouldCallGetHasUserNoOrganisationsEinheitIdHint() {
			doReturn(Optional.empty()).when(service).getHasUserNoOrganisationsEinheitIdHint();

			service.findHints();

			verify(service).getHasUserNoOrganisationsEinheitIdHint();
		}

		@Test
		void shouldReturnHintsIfPresent() {
			var hint = HintTestFactory.create();
			doReturn(Optional.of(hint)).when(service).getHasUserNoOrganisationsEinheitIdHint();

			var hints = service.findHints();

			assertThat(hints).containsExactly(hint);
		}

		@Test
		void shouldReturnEmptyListIfPresent() {
			doReturn(Optional.empty()).when(service).getHasUserNoOrganisationsEinheitIdHint();

			var hints = service.findHints();

			assertThat(hints).isEmpty();
		}
	}

	@DisplayName("GetHasUserNoOrganisationsEinheitIdHint")
	@Nested
	class TestGetHasNoOrganisationsEinheitIdHint {

		@Test
		void shouldCallHasOrganisationsEinheitId() {
			doReturn(Collections.emptyList()).when(service).getOrganisationsEinheitIds();

			service.getHasUserNoOrganisationsEinheitIdHint();

			verify(service).getOrganisationsEinheitIds();
		}

		@Test
		void shouldCallIsMissingMinimumRoles() {
			doReturn(Collections.emptyList()).when(service).getOrganisationsEinheitIds();

			service.getHasUserNoOrganisationsEinheitIdHint();

			verify(service).hasOrganisationsEinheitIdRequiredRole();
		}

		@Test
		void shouldReturnHintIfConditionMatch() {
			doReturn(Collections.emptyList()).when(service).getOrganisationsEinheitIds();
			doReturn(true).when(service).hasOrganisationsEinheitIdRequiredRole();

			var hint = service.getHasUserNoOrganisationsEinheitIdHint();

			assertThat(hint).isPresent().get().extracting(Hint::getCode).isEqualTo(HintService.USER_NO_ORGANISATIONS_EINHEIT_ID_CODE);
		}

		@Test
		void shouldReturnEmptyIsConditionNotMatched() {
			doReturn(List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID)).when(service).getOrganisationsEinheitIds();

			var hint = service.getHasUserNoOrganisationsEinheitIdHint();

			assertThat(hint).isNotPresent();
		}

		@DisplayName("Has OrganisationsEinheitId")
		@Nested
		class TestHasOrganisationsEinheitId {

			@BeforeEach
			void mockUser() {
				when(currentUserService.getUser()).thenReturn(UserProfileTestFactory.create());
			}

			@Test
			void shouldGetUser() {
				service.getOrganisationsEinheitIds();

				verify(currentUserService).getUser();
			}

			@Test
			void shouldReturnTrueIfIsNotEmpty() {
				var hasOrganisationsEinheitId = service.getOrganisationsEinheitIds();

				assertThat(hasOrganisationsEinheitId).isEqualTo(List.of(ZustaendigeStelleTestFactory.ORGANISATIONSEINHEITEN_ID));

			}

			@Test
			void shouldReturnFalseIfIsEmpty() {
				when(currentUserService.getUser()).thenReturn(UserProfileTestFactory.createBuilder().clearOrganisationseinheitIds().build());

				var hasOrganisationsEinheitId = service.getOrganisationsEinheitIds();

				assertThat(hasOrganisationsEinheitId).isEmpty();
			}
		}

		@DisplayName("Has organisationsEinheitId required role")
		@Nested
		class TestHasOrganisationsEinheitIdRelatedRole {

			@Test
			void shouldCheckEinheitlicherAnsprechpartnerRole() {
				service.hasOrganisationsEinheitIdRequiredRole();

				verify(currentUserService).hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER);
			}

			@Test
			void shouldCheckVerwaltungPoststelleRole() {
				when(currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(false);

				service.hasOrganisationsEinheitIdRequiredRole();

				verify(currentUserService).hasRole(UserRole.VERWALTUNG_POSTSTELLE);
			}

			@Test
			void shouldReturnFalseOnVerwaltungPoststelle() {
				when(currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(false);
				when(currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(true);

				var hasRequiredRole = service.hasOrganisationsEinheitIdRequiredRole();

				assertThat(hasRequiredRole).isFalse();
			}

			@Test
			void shouldReturnFalseOnEinheitlicherAnsprechpartner() {
				when(currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(true);

				var hasRequiredRole = service.hasOrganisationsEinheitIdRequiredRole();

				assertThat(hasRequiredRole).isFalse();
			}

			@Test
			void shouldReturnFalseOnOtherRole() {
				when(currentUserService.hasRole(UserRole.EINHEITLICHER_ANSPRECHPARTNER)).thenReturn(false);
				when(currentUserService.hasRole(UserRole.VERWALTUNG_POSTSTELLE)).thenReturn(false);

				var hasRequiredRole = service.hasOrganisationsEinheitIdRequiredRole();

				assertThat(hasRequiredRole).isTrue();
			}
		}

		@DisplayName("Has hints")
		@Nested
		class TestHasHints {

			@Test
			void shouldCallFindHints() {
				doReturn(Collections.emptyList()).when(service).findHints();

				service.hasHints();

				verify(service).findHints();
			}

			@Test
			void shouldReturnFalseIfNoHintsFound() {
				doReturn(List.of(HintTestFactory.create())).when(service).findHints();

				var hasHints = service.hasHints();

				assertThat(hasHints).isTrue();
			}

			@Test
			void shouldReturnTrueIfHintsFound() {
				doReturn(Collections.emptyList()).when(service).findHints();

				var hasHints = service.hasHints();

				assertThat(hasHints).isFalse();
			}
		}
	}
}
