/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.postfach;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.hamcrest.core.StringContains;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.command.CommandController;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.postfach.PostfachMailController.PostfachMailCommandByVorgangController;
import de.ozgcloud.alfa.vorgang.VorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangWithEingangTestFactory;
import lombok.SneakyThrows;

class PostfachMailCommandByVorgangControllerTest {

	@Spy
	@InjectMocks // NOSONAR
	private PostfachMailCommandByVorgangController controller;
	@Mock
	private PostfachMailService service;
	@Mock
	private PostfachNachrichtHelper helper;
	@Mock
	private VorgangController vorgangController;

	private MockMvc mockMvc;

	@BeforeEach
	void initTest() {
		mockMvc = MockMvcBuilders.standaloneSetup(controller).build();
	}

	@DisplayName("Create command")
	@Nested
	class TestCreateCommand {

		@Captor
		private ArgumentCaptor<CreateCommand> createCommandCaptor;
		@Captor
		private ArgumentCaptor<Long> relationVersionCaptor;

		@BeforeEach
		void mockVorgangController() {
			when(vorgangController.getVorgang(anyString())).thenReturn(VorgangWithEingangTestFactory.create());
			when(service.sendPostfachMail(any(), anyLong())).thenReturn(CommandTestFactory.create());
			when(helper.addPostfachAddressToBody(any(), any())).thenReturn(PostfachMailTestFactory.create());
		}

		@Test
		void shouldSetVorgangId() {
			doRequest();

			assertThat(createCommandCaptor.getValue().getVorgangId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldSetRelationId() {
			doRequest();

			assertThat(createCommandCaptor.getValue().getRelationId()).isEqualTo(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallVorgangController() {
			doRequest();

			verify(vorgangController).getVorgang(VorgangHeaderTestFactory.ID);
		}

		@Test
		void shouldCallHelper() {
			doRequest();

			verify(helper).addPostfachAddressToBody(any(), any());
		}

		@Test
		void shouldAddPostfachAddress() {
			doRequest();

			assertThat(getCommandBody().getPostfachAddress()).usingRecursiveComparison().isEqualTo(PostfachAddressTestFactory.createAsMap());
		}

		@Test
		void shouldCallService() {
			doRequest();

			verify(service).sendPostfachMail(any(CreateCommand.class), eq(VorgangHeaderTestFactory.VERSION));
		}

		@Test
		void shouldProceedWithEmptyAttachments() {
			when(helper.addPostfachAddressToBody(any(), any())).thenReturn(PostfachMailTestFactory.createBuilder().clearAttachments().build());

			doRequest(PostfachMailTestFactory.createBuilder().clearAttachments().build());

			assertThat(getCommandBody().getAttachments()).isEmpty();
		}

		private PostfachMail getCommandBody() {
			return (PostfachMail) createCommandCaptor.getValue().getBody();
		}

		@SneakyThrows
		@Test
		void shouldReturnLocationHeader() {
			var response = doRequest();

			response.andExpect(
					header().string("Location", StringContains.containsString(CommandController.COMMANDS_PATH + "/" + CommandTestFactory.ID)));
		}

		private ResultActions doRequest() {
			return doRequest(PostfachMailTestFactory.create());
		}

		@SneakyThrows
		private ResultActions doRequest(PostfachMail postfachMail) {
			var response = mockMvc.perform(post(PostfachMailCommandByVorgangController.PATH.replace("{vorgangId}", VorgangHeaderTestFactory.ID))
					.contentType(MediaType.APPLICATION_JSON)
					.content(PostfachMailTestFactory.buildSendPostfachMailContent(postfachMail)))
					.andExpect(status().isCreated());

			verify(service).sendPostfachMail(createCommandCaptor.capture(), relationVersionCaptor.capture());

			return response;
		}
	}
}